# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.services.Service
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(concrete.services.Service.Iface):
    """
    The active learning server is responsible for sorting a list of communications.
    Users annotate communications based on the sort.

    Active learning is an asynchronous process.
    It is started by the client calling start().
    At arbitrary times, the client can call addAnnotations().
    When the server is done with a sort of the data, it calls submitSort() on the client.
    The server can perform additional sorts until stop() is called.

    The server must be preconfigured with the details of the data source to pull communications.
    """
    def start(self, sessionId, task, contact):
        """
        Start an active learning session on these communications

        Parameters:
         - sessionId
         - task
         - contact
        """
        pass

    def stop(self, sessionId):
        """
        Stop the learning session

        Parameters:
         - sessionId
        """
        pass

    def addAnnotations(self, sessionId, annotations):
        """
        Add annotations from the user to the learning process

        Parameters:
         - sessionId
         - annotations
        """
        pass


class Client(concrete.services.Service.Client, Iface):
    """
    The active learning server is responsible for sorting a list of communications.
    Users annotate communications based on the sort.

    Active learning is an asynchronous process.
    It is started by the client calling start().
    At arbitrary times, the client can call addAnnotations().
    When the server is done with a sort of the data, it calls submitSort() on the client.
    The server can perform additional sorts until stop() is called.

    The server must be preconfigured with the details of the data source to pull communications.
    """
    def __init__(self, iprot, oprot=None):
        concrete.services.Service.Client.__init__(self, iprot, oprot)

    def start(self, sessionId, task, contact):
        """
        Start an active learning session on these communications

        Parameters:
         - sessionId
         - task
         - contact
        """
        self.send_start(sessionId, task, contact)
        return self.recv_start()

    def send_start(self, sessionId, task, contact):
        self._oprot.writeMessageBegin('start', TMessageType.CALL, self._seqid)
        args = start_args()
        args.sessionId = sessionId
        args.task = task
        args.contact = contact
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_start(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = start_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "start failed: unknown result")

    def stop(self, sessionId):
        """
        Stop the learning session

        Parameters:
         - sessionId
        """
        self.send_stop(sessionId)
        self.recv_stop()

    def send_stop(self, sessionId):
        self._oprot.writeMessageBegin('stop', TMessageType.CALL, self._seqid)
        args = stop_args()
        args.sessionId = sessionId
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_stop(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = stop_result()
        result.read(iprot)
        iprot.readMessageEnd()
        return

    def addAnnotations(self, sessionId, annotations):
        """
        Add annotations from the user to the learning process

        Parameters:
         - sessionId
         - annotations
        """
        self.send_addAnnotations(sessionId, annotations)
        self.recv_addAnnotations()

    def send_addAnnotations(self, sessionId, annotations):
        self._oprot.writeMessageBegin('addAnnotations', TMessageType.CALL, self._seqid)
        args = addAnnotations_args()
        args.sessionId = sessionId
        args.annotations = annotations
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_addAnnotations(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = addAnnotations_result()
        result.read(iprot)
        iprot.readMessageEnd()
        return


class Processor(concrete.services.Service.Processor, Iface, TProcessor):
    def __init__(self, handler):
        concrete.services.Service.Processor.__init__(self, handler)
        self._processMap["start"] = Processor.process_start
        self._processMap["stop"] = Processor.process_stop
        self._processMap["addAnnotations"] = Processor.process_addAnnotations

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_start(self, seqid, iprot, oprot):
        args = start_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = start_result()
        try:
            result.success = self._handler.start(args.sessionId, args.task, args.contact)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("start", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_stop(self, seqid, iprot, oprot):
        args = stop_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = stop_result()
        try:
            self._handler.stop(args.sessionId)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("stop", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_addAnnotations(self, seqid, iprot, oprot):
        args = addAnnotations_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = addAnnotations_result()
        try:
            self._handler.addAnnotations(args.sessionId, args.annotations)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("addAnnotations", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class start_args(object):
    """
    Attributes:
     - sessionId
     - task
     - contact
    """


    def __init__(self, sessionId=None, task=None, contact=None,):
        self.sessionId = sessionId
        self.task = task
        self.contact = contact

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionId = concrete.uuid.ttypes.UUID()
                    self.sessionId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.task = AnnotationTask()
                    self.task.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    self.contact = concrete.services.ttypes.AsyncContactInfo()
                    self.contact.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('start_args')
        if self.sessionId is not None:
            oprot.writeFieldBegin('sessionId', TType.STRUCT, 1)
            self.sessionId.write(oprot)
            oprot.writeFieldEnd()
        if self.task is not None:
            oprot.writeFieldBegin('task', TType.STRUCT, 2)
            self.task.write(oprot)
            oprot.writeFieldEnd()
        if self.contact is not None:
            oprot.writeFieldBegin('contact', TType.STRUCT, 3)
            self.contact.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(start_args)
start_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRUCT, 'task', [AnnotationTask, None], None, ),  # 2
    (3, TType.STRUCT, 'contact', [concrete.services.ttypes.AsyncContactInfo, None], None, ),  # 3
)


class start_result(object):
    """
    Attributes:
     - success
    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.BOOL:
                    self.success = iprot.readBool()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('start_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.BOOL, 0)
            oprot.writeBool(self.success)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(start_result)
start_result.thrift_spec = (
    (0, TType.BOOL, 'success', None, None, ),  # 0
)


class stop_args(object):
    """
    Attributes:
     - sessionId
    """


    def __init__(self, sessionId=None,):
        self.sessionId = sessionId

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionId = concrete.uuid.ttypes.UUID()
                    self.sessionId.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('stop_args')
        if self.sessionId is not None:
            oprot.writeFieldBegin('sessionId', TType.STRUCT, 1)
            self.sessionId.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(stop_args)
stop_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
)


class stop_result(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('stop_result')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(stop_result)
stop_result.thrift_spec = (
)


class addAnnotations_args(object):
    """
    Attributes:
     - sessionId
     - annotations
    """


    def __init__(self, sessionId=None, annotations=None,):
        self.sessionId = sessionId
        self.annotations = annotations

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionId = concrete.uuid.ttypes.UUID()
                    self.sessionId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.annotations = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = Annotation()
                        _elem12.read(iprot)
                        self.annotations.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('addAnnotations_args')
        if self.sessionId is not None:
            oprot.writeFieldBegin('sessionId', TType.STRUCT, 1)
            self.sessionId.write(oprot)
            oprot.writeFieldEnd()
        if self.annotations is not None:
            oprot.writeFieldBegin('annotations', TType.LIST, 2)
            oprot.writeListBegin(TType.STRUCT, len(self.annotations))
            for iter13 in self.annotations:
                iter13.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(addAnnotations_args)
addAnnotations_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.LIST, 'annotations', (TType.STRUCT, [Annotation, None], False), None, ),  # 2
)


class addAnnotations_result(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('addAnnotations_result')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(addAnnotations_result)
addAnnotations_result.thrift_spec = (
)
fix_spec(all_structs)
del all_structs




# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys

from thrift.transport import TTransport
all_structs = []


class EmailAddress(object):
    """
    An email address, optionally accompanied by a display_name. These
    values are typically extracted from strings such as:
    <tt> "John Smith" &lt;john\@xyz.com&gt; </tt>.

    \see RFC2822 http://tools.ietf.org/html/rfc2822

    Attributes:
     - address
     - displayName
    """


    def __init__(self, address=None, displayName=None,):
        self.address = address
        self.displayName = displayName

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.address = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.displayName = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('EmailAddress')
        if self.address is not None:
            oprot.writeFieldBegin('address', TType.STRING, 1)
            oprot.writeString(self.address.encode('utf-8') if sys.version_info[0] == 2 else self.address)
            oprot.writeFieldEnd()
        if self.displayName is not None:
            oprot.writeFieldBegin('displayName', TType.STRING, 2)
            oprot.writeString(self.displayName.encode('utf-8') if sys.version_info[0] == 2 else self.displayName)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class EmailCommunicationInfo(object):
    """
    Extra information about an email communication instance.

    Attributes:
     - messageId
     - contentType
     - userAgent
     - inReplyToList
     - referenceList
     - senderAddress
     - returnPathAddress
     - toAddressList
     - ccAddressList
     - bccAddressList
     - emailFolder
     - subject
     - quotedAddresses
     - attachmentPaths
     - salutation
     - signature
    """


    def __init__(self, messageId=None, contentType=None, userAgent=None, inReplyToList=None, referenceList=None, senderAddress=None, returnPathAddress=None, toAddressList=None, ccAddressList=None, bccAddressList=None, emailFolder=None, subject=None, quotedAddresses=None, attachmentPaths=None, salutation=None, signature=None,):
        self.messageId = messageId
        self.contentType = contentType
        self.userAgent = userAgent
        self.inReplyToList = inReplyToList
        self.referenceList = referenceList
        self.senderAddress = senderAddress
        self.returnPathAddress = returnPathAddress
        self.toAddressList = toAddressList
        self.ccAddressList = ccAddressList
        self.bccAddressList = bccAddressList
        self.emailFolder = emailFolder
        self.subject = subject
        self.quotedAddresses = quotedAddresses
        self.attachmentPaths = attachmentPaths
        self.salutation = salutation
        self.signature = signature

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.messageId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.contentType = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.userAgent = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.inReplyToList = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.inReplyToList.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.referenceList = []
                    (_etype9, _size6) = iprot.readListBegin()
                    for _i10 in range(_size6):
                        _elem11 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.referenceList.append(_elem11)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRUCT:
                    self.senderAddress = EmailAddress()
                    self.senderAddress.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.STRUCT:
                    self.returnPathAddress = EmailAddress()
                    self.returnPathAddress.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.LIST:
                    self.toAddressList = []
                    (_etype15, _size12) = iprot.readListBegin()
                    for _i16 in range(_size12):
                        _elem17 = EmailAddress()
                        _elem17.read(iprot)
                        self.toAddressList.append(_elem17)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.LIST:
                    self.ccAddressList = []
                    (_etype21, _size18) = iprot.readListBegin()
                    for _i22 in range(_size18):
                        _elem23 = EmailAddress()
                        _elem23.read(iprot)
                        self.ccAddressList.append(_elem23)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 10:
                if ftype == TType.LIST:
                    self.bccAddressList = []
                    (_etype27, _size24) = iprot.readListBegin()
                    for _i28 in range(_size24):
                        _elem29 = EmailAddress()
                        _elem29.read(iprot)
                        self.bccAddressList.append(_elem29)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 11:
                if ftype == TType.STRING:
                    self.emailFolder = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 12:
                if ftype == TType.STRING:
                    self.subject = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 13:
                if ftype == TType.LIST:
                    self.quotedAddresses = []
                    (_etype33, _size30) = iprot.readListBegin()
                    for _i34 in range(_size30):
                        _elem35 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.quotedAddresses.append(_elem35)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 14:
                if ftype == TType.LIST:
                    self.attachmentPaths = []
                    (_etype39, _size36) = iprot.readListBegin()
                    for _i40 in range(_size36):
                        _elem41 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.attachmentPaths.append(_elem41)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 15:
                if ftype == TType.STRING:
                    self.salutation = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 16:
                if ftype == TType.STRING:
                    self.signature = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('EmailCommunicationInfo')
        if self.messageId is not None:
            oprot.writeFieldBegin('messageId', TType.STRING, 1)
            oprot.writeString(self.messageId.encode('utf-8') if sys.version_info[0] == 2 else self.messageId)
            oprot.writeFieldEnd()
        if self.contentType is not None:
            oprot.writeFieldBegin('contentType', TType.STRING, 2)
            oprot.writeString(self.contentType.encode('utf-8') if sys.version_info[0] == 2 else self.contentType)
            oprot.writeFieldEnd()
        if self.userAgent is not None:
            oprot.writeFieldBegin('userAgent', TType.STRING, 3)
            oprot.writeString(self.userAgent.encode('utf-8') if sys.version_info[0] == 2 else self.userAgent)
            oprot.writeFieldEnd()
        if self.inReplyToList is not None:
            oprot.writeFieldBegin('inReplyToList', TType.LIST, 4)
            oprot.writeListBegin(TType.STRING, len(self.inReplyToList))
            for iter42 in self.inReplyToList:
                oprot.writeString(iter42.encode('utf-8') if sys.version_info[0] == 2 else iter42)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.referenceList is not None:
            oprot.writeFieldBegin('referenceList', TType.LIST, 5)
            oprot.writeListBegin(TType.STRING, len(self.referenceList))
            for iter43 in self.referenceList:
                oprot.writeString(iter43.encode('utf-8') if sys.version_info[0] == 2 else iter43)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.senderAddress is not None:
            oprot.writeFieldBegin('senderAddress', TType.STRUCT, 6)
            self.senderAddress.write(oprot)
            oprot.writeFieldEnd()
        if self.returnPathAddress is not None:
            oprot.writeFieldBegin('returnPathAddress', TType.STRUCT, 7)
            self.returnPathAddress.write(oprot)
            oprot.writeFieldEnd()
        if self.toAddressList is not None:
            oprot.writeFieldBegin('toAddressList', TType.LIST, 8)
            oprot.writeListBegin(TType.STRUCT, len(self.toAddressList))
            for iter44 in self.toAddressList:
                iter44.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ccAddressList is not None:
            oprot.writeFieldBegin('ccAddressList', TType.LIST, 9)
            oprot.writeListBegin(TType.STRUCT, len(self.ccAddressList))
            for iter45 in self.ccAddressList:
                iter45.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.bccAddressList is not None:
            oprot.writeFieldBegin('bccAddressList', TType.LIST, 10)
            oprot.writeListBegin(TType.STRUCT, len(self.bccAddressList))
            for iter46 in self.bccAddressList:
                iter46.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.emailFolder is not None:
            oprot.writeFieldBegin('emailFolder', TType.STRING, 11)
            oprot.writeString(self.emailFolder.encode('utf-8') if sys.version_info[0] == 2 else self.emailFolder)
            oprot.writeFieldEnd()
        if self.subject is not None:
            oprot.writeFieldBegin('subject', TType.STRING, 12)
            oprot.writeString(self.subject.encode('utf-8') if sys.version_info[0] == 2 else self.subject)
            oprot.writeFieldEnd()
        if self.quotedAddresses is not None:
            oprot.writeFieldBegin('quotedAddresses', TType.LIST, 13)
            oprot.writeListBegin(TType.STRING, len(self.quotedAddresses))
            for iter47 in self.quotedAddresses:
                oprot.writeString(iter47.encode('utf-8') if sys.version_info[0] == 2 else iter47)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.attachmentPaths is not None:
            oprot.writeFieldBegin('attachmentPaths', TType.LIST, 14)
            oprot.writeListBegin(TType.STRING, len(self.attachmentPaths))
            for iter48 in self.attachmentPaths:
                oprot.writeString(iter48.encode('utf-8') if sys.version_info[0] == 2 else iter48)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.salutation is not None:
            oprot.writeFieldBegin('salutation', TType.STRING, 15)
            oprot.writeString(self.salutation.encode('utf-8') if sys.version_info[0] == 2 else self.salutation)
            oprot.writeFieldEnd()
        if self.signature is not None:
            oprot.writeFieldBegin('signature', TType.STRING, 16)
            oprot.writeString(self.signature.encode('utf-8') if sys.version_info[0] == 2 else self.signature)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(EmailAddress)
EmailAddress.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'address', 'UTF8', None, ),  # 1
    (2, TType.STRING, 'displayName', 'UTF8', None, ),  # 2
)
all_structs.append(EmailCommunicationInfo)
EmailCommunicationInfo.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'messageId', 'UTF8', None, ),  # 1
    (2, TType.STRING, 'contentType', 'UTF8', None, ),  # 2
    (3, TType.STRING, 'userAgent', 'UTF8', None, ),  # 3
    (4, TType.LIST, 'inReplyToList', (TType.STRING, 'UTF8', False), None, ),  # 4
    (5, TType.LIST, 'referenceList', (TType.STRING, 'UTF8', False), None, ),  # 5
    (6, TType.STRUCT, 'senderAddress', [EmailAddress, None], None, ),  # 6
    (7, TType.STRUCT, 'returnPathAddress', [EmailAddress, None], None, ),  # 7
    (8, TType.LIST, 'toAddressList', (TType.STRUCT, [EmailAddress, None], False), None, ),  # 8
    (9, TType.LIST, 'ccAddressList', (TType.STRUCT, [EmailAddress, None], False), None, ),  # 9
    (10, TType.LIST, 'bccAddressList', (TType.STRUCT, [EmailAddress, None], False), None, ),  # 10
    (11, TType.STRING, 'emailFolder', 'UTF8', None, ),  # 11
    (12, TType.STRING, 'subject', 'UTF8', None, ),  # 12
    (13, TType.LIST, 'quotedAddresses', (TType.STRING, 'UTF8', False), None, ),  # 13
    (14, TType.LIST, 'attachmentPaths', (TType.STRING, 'UTF8', False), None, ),  # 14
    (15, TType.STRING, 'salutation', 'UTF8', None, ),  # 15
    (16, TType.STRING, 'signature', 'UTF8', None, ),  # 16
)
fix_spec(all_structs)
del all_structs



from __future__ import unicode_literals
import time

from concrete import AnnotationMetadata, Tokenization, UUID
from test_helper import read_test_comm


def test_repr_on_comm():
    """Verify that Communications can be converted to strings.

    Checks for the issue addressed in this commit:

    commit 0ee3317454543b63dc7a273d92e5720bb9210b03
    Author: Craig Harman <craig@craigharman.net>
    Date:   Tue Dec 16 13:08:44 2014 -0500

    Fixed infinite recursion bug in Tokenization.__repr__()

    The addition of an in-memory "backpointer" from a Tokenization to the
    Tokenization's enclosing Sentence inadvertently broke the
    (Thrift auto-generated) Tokenization.__repr__() function.  Modified the
    function to ignore the backpointer when generating the string
    representation for a Tokenization.
    """
    comm = read_test_comm()
    comm.__repr__()


def test_repr_on_tokenization():
    tokenization = Tokenization(
        metadata=AnnotationMetadata(
            tool="test", timestamp=int(time.time())),
        uuid=UUID(uuidString='01234567-0123-4567-89ab-cdef89abcdef')
    )
    tokenization.__repr__()



"""Convert between JSON and Concrete representations of Tweets

The JSON fields used by the Twitter API are documented at:

  https://dev.twitter.com/overview/api/tweets
"""
from __future__ import unicode_literals

import json
import logging
import time
import pycountry
from datetime import datetime

from ..metadata.ttypes import AnnotationMetadata, CommunicationMetadata
from ..communication.ttypes import Communication
from ..language.ttypes import LanguageIdentification
from ..twitter.ttypes import (
    BoundingBox,
    HashTag,
    PlaceAttributes,
    TweetInfo,
    TwitterCoordinates,
    TwitterEntities,
    TwitterLatLong,
    TwitterPlace,
    TwitterUser,
    URL,
    UserMention
)

from .concrete_uuid import AnalyticUUIDGeneratorFactory
from .metadata import datetime_to_timestamp


TOOL_NAME = "Python module concrete.util.twitter"
TWEET_TYPE = "Tweet"
ISO_LANGS = pycountry.languages
CREATED_AT_FORMAT = '%a %b %d %H:%M:%S +0000 %Y'


def json_tweet_object_to_Communication(tweet):
    """Convert deserialized JSON Tweet object to :class:`.Communication`

    Args:
        tweet (object): Object created by deserializing a JSON Tweet string

    Returns:
        Communication: Communication representing the Tweet, with
        `tweetInfo` and `text` fields set (among others) but with
        a null (None) `sectionList`.
    """
    tweet_info = json_tweet_object_to_TweetInfo(tweet)

    augf = AnalyticUUIDGeneratorFactory()
    aug = augf.create()
    if 'id_str' in tweet:
        tweet_id = tweet['id_str']
    else:
        logging.warning('Tweet has no id_str, leaving communication id blank')
        tweet_id = None
    tweet_time = datetime_to_timestamp(datetime.strptime(tweet_info.createdAt,
                                                         CREATED_AT_FORMAT))
    comm = Communication(
        communicationMetadata=CommunicationMetadata(
            tweetInfo=tweet_info),
        metadata=AnnotationMetadata(
            tool=TOOL_NAME,
            timestamp=int(time.time())),
        originalText=json.dumps(tweet),
        text=tweet_info.text,
        type=TWEET_TYPE,
        uuid=next(aug),
        startTime=tweet_time,
        endTime=tweet_time,
        id=tweet_id
    )

    # either this, or pass in gen as parameter to fx
    # latter is more annoying to test but slightly cleaner
    if tweet_info.lid is not None:
        tweet_info.lid.uuid = next(aug)
        lidList = [tweet_info.lid]
        comm.lidList = lidList
    return comm


def snake_case_to_camelcase(value):
    """Converts snake case to camel case

    Implementation copied from this Stack Overflow post:
    http://goo.gl/SSgo9k

    Args:
        value (str): snake case (lower case with underscores) value

    Returns:
        str: camel case string corresponding to value (with isolated
        unscores stripped and sequences of two or more underscores
        reduced by one underscore)
    """
    def camelcase():
        yield lambda c: c.lower()
        while True:
            yield lambda c: c.capitalize()
    c = iter(camelcase())
    return u"".join(next(c)(x) if x else u'_' for x in value.split(u"_"))


def json_tweet_object_to_TweetInfo(tweet):
    """Create :class:`.TweetInfo` object from deserialized JSON Tweet
    object

    Args:
        tweet (dict): Object created by deserializing a JSON Tweet
            string

    Returns:
        TweetInfo: concrete object representing twitter metadata from
        tweet
    """

    def set_flat_fields(concrete_object, twitter_dict):
        """Copy data from the dictionary for the Twitter object to the
        corresponding Concrete data structure, ignoring nested
        data structures.

        The Twitter API uses snake_case for field names while the Concrete
        schema uses CamelCase for the same fields.

        Args:
            concrete_object (object): concrete object whose fields will
                be set according to the content in twitter_dict;
                will be modified
            twitter_dict (dict): twitter object (dictionary) whose
                entries will be used to set fields on concrete_object
        """
        for key in twitter_dict.keys():
            if type(twitter_dict[key]) != dict:
                camelcased_key = snake_case_to_camelcase(key)
                if hasattr(concrete_object, camelcased_key):
                    setattr(concrete_object, camelcased_key, twitter_dict[key])
                else:
                    logging.debug("Concrete schema for '%s' missing field "
                                  "for Twitter API field '%s'" %
                                  (type(concrete_object), key))

    tweet_info = TweetInfo()
    set_flat_fields(tweet_info, tweet)

    twitter_user = TwitterUser()
    json_twitter_user = tweet[u'user']
    set_flat_fields(twitter_user, json_twitter_user)
    if json_twitter_user['lang']:
        twitter_lid = json_twitter_user['lang']
        twitter_user.lang = twitter_lid_to_iso639_3(twitter_lid)
    tweet_info.user = twitter_user

    if u'retweeted_status' in tweet and tweet[u'retweeted_status']:
        tweet_info.retweetedStatusId = tweet[u'retweeted_status']['id']
        tweet_info.retweetedUserId = tweet[u'retweeted_status']['user']['id']
        tweet_info.retweetedScreenName = tweet[u'retweeted_status']['user'][
            'screen_name'
        ]

    if u'entities' in tweet and tweet[u'entities']:
        twitter_entities = TwitterEntities()
        if tweet[u'entities'][u'hashtags']:
            twitter_entities.hashtagList = []
            for hashtag_dict in tweet[u'entities'][u'hashtags']:
                hashtag = HashTag()
                set_flat_fields(hashtag, hashtag_dict)
                hashtag.startOffset = hashtag_dict[u'indices'][0]
                hashtag.endOffset = hashtag_dict[u'indices'][1]
                twitter_entities.hashtagList.append(hashtag)
        if tweet[u'entities'][u'urls']:
            twitter_entities.urlList = []
            for url_dict in tweet[u'entities'][u'urls']:
                url = URL()
                set_flat_fields(url, url_dict)
                url.startOffset = url_dict[u'indices'][0]
                url.endOffset = url_dict[u'indices'][1]
                twitter_entities.urlList.append(url)
        if tweet[u'entities'][u'user_mentions']:
            twitter_entities.userMentionList = []
            for user_mention_dict in tweet[u'entities'][u'user_mentions']:
                user_mention = UserMention()
                set_flat_fields(user_mention, user_mention_dict)
                user_mention.startOffset = user_mention_dict[u'indices'][0]
                user_mention.endOffset = user_mention_dict[u'indices'][1]
                twitter_entities.userMentionList.append(user_mention)
        tweet_info.entities = twitter_entities

    if u'coordinates' in tweet and tweet[u'coordinates']:
        tweet_coordinates = TwitterCoordinates()
        tweet_coordinates.type = tweet[u'coordinates']['type']
        tweet_coordinates.coordinates = TwitterLatLong(
            longitude=tweet[u'coordinates'][u'coordinates'][0],
            latitude=tweet[u'coordinates'][u'coordinates'][1])
        tweet_info.coordinates = tweet_coordinates

    if u'place' in tweet and tweet[u'place']:
        twitter_place = TwitterPlace()
        set_flat_fields(twitter_place, tweet[u'place'])
        if tweet[u'place'][u'bounding_box']:
            bounding_box = BoundingBox()
            bb_dict = tweet[u'place'][u'bounding_box']
            set_flat_fields(bounding_box, bb_dict)
            if bounding_box.coordinateList is None:
                bounding_box.coordinateList = []
            if u'coordinates' in bb_dict and bb_dict[u'coordinates']:
                for [longitude, latitude] in bb_dict[u'coordinates'][0]:
                    bounding_box.coordinateList.append(TwitterLatLong(
                        longitude=longitude,
                        latitude=latitude))
            twitter_place.boundingBox = bounding_box
        if tweet[u'place'][u'attributes']:
            place_attributes = PlaceAttributes()
            set_flat_fields(place_attributes,
                            tweet[u'place'][u'attributes'])
            twitter_place.attributes = place_attributes
        tweet_info.place = twitter_place

    tweet_info.lid = capture_tweet_lid(tweet)
    return tweet_info


def json_tweet_string_to_Communication(json_tweet_string, check_empty=False,
                                       check_delete=False):
    """Convert JSON Tweet string to Communication

    Args:
        json_tweet_string (str): JSON Tweet string from Twitter API
        check_empty (bool): If `True`, check if `json_tweet_string` is empty
            (return None if it is)
        check_delete (bool): If `True`, check for presence of `delete` field
            in Tweet JSON, and if the 'delete' field is present, return `None`

    Returns:
        Communication: Communication representing the Tweet, with
        `tweetInfo` and `text` fields set (among others) but with
        a null (None) `sectionList`.
    """

    json_tweet_string = json_tweet_string.strip()
    if (not check_empty) or json_tweet_string:
        json_tweet = json.loads(json_tweet_string)
        if (not check_delete) or tuple(json_tweet.keys()) != ('delete',):
            return json_tweet_object_to_Communication(json_tweet)
        else:
            return None
    else:
        return None


def json_tweet_string_to_TweetInfo(json_tweet_string):
    """Create :class:`.TweetInfo` object from JSON Tweet string

    Args:
        json_tweet_string (str): JSON Tweet string from Twitter API

    Returns:
        TweetInfo: concrete twitter metadata object with fields
        set from json_tweet_string
    """
    tweet = json.loads(json_tweet_string)
    return json_tweet_object_to_TweetInfo(tweet)


def capture_tweet_lid(tweet):
    """
    Reads the `lang` field from a tweet from the twitter API, if it
    exists, and return corresponding concrete
    :class:`LanguageIdentification` object.

    Args:
        tweet (dict): Object created by deserializing a JSON Tweet string

    Returns:
        :class:`.LanguageIdentification` object, or None
        if the `lang` field is not present in the Tweet JSON
    """
    if u'lang' in tweet:
        amd = AnnotationMetadata(tool="Twitter LID",
                                 timestamp=int(time.time()),
                                 kBest=1)
        kvs = {}
        kvs[twitter_lid_to_iso639_3(tweet[u'lang'])] = 1.0
        return LanguageIdentification(metadata=amd,
                                      languageToProbabilityMap=kvs)
    else:
        return None


def twitter_lid_to_iso639_3(twitter_lid):
    """Convert Twitter Language ID string to ISO639-3 code

    Ref:
        https://developer.twitter.com/en/docs/developer-utilities/supported-languages/api-reference/get-help-languages

    Args:
        twitter_lid (str): This can be an iso639-3 code (no-op),
            iso639-1 2-letter abbr (converted to 3), or combo
            (split by '-', then first part converted)

            Per the Twitter documentation, "The language code may be
            formatted as ISO 639-1 alpha-2 (en), ISO 639-3 alpha-3
            (msa), or ISO 639-1 alpha-2 combined with an ISO 3166-1
            alpha-2 localization (zh-tw)."

    Returns:
        str: the ISO639-3 code corresponding to twitter_lid

    """
    def _iso639_1_to_iso639_3(iso639_1):
        if iso639_1 == 'in':
            return ISO_LANGS.get(alpha_2='id').alpha_3

        try:
            # pycountry 18.12.8 changed the behavior of pycountry.languages.get().
            # Prior versions raised a KeyError if the language was not found, but
            # starting with pycountry 18.12.8, None is returned instead:
            #   https://pypi.org/project/pycountry/18.12.8/
            iso_lang = ISO_LANGS.get(alpha_2=iso639_1)
            if iso_lang:
                return iso_lang.alpha_3
            else:
                return 'und'
        except KeyError:
            # As of early 2018, Twitter is (at least sometimes) using
            # the incorrect ISO-639-1 language code for Indonesian.
            #
            # This issue was acknowledged by Twitter tech support in this
            # 2015 forum post:
            #   https://twittercommunity.com/t/hebrew-and-indonesian-getting-incorrect-lang-codes/13044/3
            #
            # The 2015 forum post mentions that Twitter is using the incorrect
            # language code for both Hebrew and Indonesian, but as of 2018
            # Twitter is using the correct ISO-639-1 code for Hebrew.
            #
            # The list of Twitter language codes (including the incorrect
            # Indonesian code) could be found here:
            #   https://developer.twitter.com/en/docs/tweets/filter-realtime/guides/premium-operators
            #   http://support.gnip.com/sources/twitter/powertrack_operators.html
            # The ISO-639-1 character codes are correct for all 60+ languages currently
            # supported by Twitter, except for Indonesian.
            return 'und'

    if len(twitter_lid) == 2:
        return _iso639_1_to_iso639_3(twitter_lid)
    elif twitter_lid.find('-') == 2 and len(twitter_lid) == 5:
        return _iso639_1_to_iso639_3(twitter_lid[0:2])
    else:
        return twitter_lid



from __future__ import unicode_literals

from concrete.util import (
    read_communication_from_buffer,
    write_communication_to_buffer,
    communication_deep_copy
)
from concrete.util import create_comm
from concrete import Token


def assert_simple_comms_equal(comm1, comm2):
    assert comm1.id == comm2.id
    assert comm1.uuid.uuidString == comm2.uuid.uuidString
    assert comm1.metadata.tool == comm2.metadata.tool
    assert comm1.metadata.timestamp == comm2.metadata.timestamp
    assert (
        comm1.sectionList[0].uuid.uuidString ==
        comm2.sectionList[0].uuid.uuidString
    )
    assert (
        comm1.sectionList[0].kind ==
        comm2.sectionList[0].kind
    )
    assert (
        comm1.sectionList[0].sentenceList[0].uuid.uuidString ==
        comm2.sectionList[0].sentenceList[0].uuid.uuidString
    )
    assert (
        comm1.sectionList[0].sentenceList[0].tokenization.uuid ==
        comm2.sectionList[0].sentenceList[0].tokenization.uuid
    )
    assert (
        comm1.sectionList[0].sentenceList[0].tokenization.kind ==
        comm2.sectionList[0].sentenceList[0].tokenization.kind
    )
    assert (
        comm1.sectionList[0].sentenceList[0].tokenization.metadata.tool ==
        comm2.sectionList[0].sentenceList[0].tokenization.metadata.tool
    )
    assert (
        comm1.sectionList[0].sentenceList[0].tokenization.metadata.timestamp ==
        comm2.sectionList[0].sentenceList[0].tokenization.metadata.timestamp
    )
    assert list(map(
        lambda t: (t.text, t.tokenIndex),
        comm1.sectionList[0].sentenceList[0].tokenization.tokenList.tokenList
    )) == list(map(
        lambda t: (t.text, t.tokenIndex),
        comm2.sectionList[0].sentenceList[0].tokenization.tokenList.tokenList
    ))


def test_communication_deep_copy():
    comm1 = create_comm('a-b-c', text='foo bar baz .')
    comm2 = communication_deep_copy(comm1)
    comm3 = communication_deep_copy(comm1)
    assert_simple_comms_equal(comm1, comm2)
    assert_simple_comms_equal(comm2, comm3)
    tkzn1 = comm1.sectionList[0].sentenceList[0].tokenization
    tkzn1.tokenList.tokenList[0] = Token(text='bbq', tokenIndex=0)
    tkzn2 = comm2.sectionList[0].sentenceList[0].tokenization
    assert list(map(
        lambda t: t.text, tkzn1.tokenList.tokenList
    )) != list(map(
        lambda t: t.text, tkzn2.tokenList.tokenList
    ))
    assert_simple_comms_equal(comm2, comm3)


def test_read_against_file_contents():
    filename = u'tests/testdata/simple_1.concrete'
    with open(filename, 'rb') as f:
        buf = f.read()
        comm = read_communication_from_buffer(buf)
        assert hasattr(comm, 'sentenceForUUID')
        assert 'one' == comm.id


def test_read_against_file_contents_no_add_references():
    filename = u'tests/testdata/simple_1.concrete'
    with open(filename, 'rb') as f:
        buf = f.read()
        comm = read_communication_from_buffer(buf, add_references=False)
        assert not hasattr(comm, 'sentenceForUUID')
        assert 'one' == comm.id


def test_write_against_file_contents():
    filename = u'tests/testdata/simple_1.concrete'
    with open(filename, 'rb') as f:
        f_buf = f.read()
        comm = read_communication_from_buffer(f_buf)
    buf = write_communication_to_buffer(comm)
    assert f_buf == buf


def test_read_write_fixed_point():
    comm = create_comm('comm-1')
    buf_1 = write_communication_to_buffer(comm)
    buf_2 = write_communication_to_buffer(
        read_communication_from_buffer(buf_1)
    )
    assert buf_1 == buf_2



#!/usr/bin/env python

"""Simple webserver with Fetch/Store RPC support

A simple Bottle webserver useful for testing **concrete-js** programs
that interact with Fetch and Store servers.

Implements an HTTP server that serves static files (e.g. HTML, JS and
CSS) and provides::

- a filesystem-backed :mod:`.FetchCommunicationService`
  at URL `/fetch_http_endpoint/`

- a directory-backed :mod:`.StoreCommunicationService`
  at URL `/store_http_endpoint/`

"""

from __future__ import print_function
import argparse
import logging
import os.path
import zipfile

import bottle
import humanfriendly
from thrift.protocol import TJSONProtocol
from thrift.server import TServer
from thrift.transport import TTransport

from concrete.access import FetchCommunicationService, StoreCommunicationService
from concrete.util import (
    CommunicationContainerFetchHandler,
    DirectoryBackedStoreHandler,
    DirectoryBackedCommunicationContainer,
    MemoryBackedCommunicationContainer,
    ZipFileBackedCommunicationContainer,
    set_stdout_encoding,
)


class AccessHTTPServer(object):
    # DANGER WILL ROBINSON!  We are using class variables
    # to store values accessed by the Bottle route functions
    # below.
    FETCH_HANDLER = None
    FETCH_TSERVER = None
    STORE_HANDLER = None
    STORE_TSERVER = None
    STATIC_PATH = None

    def __init__(self, host, port, static_path, fetch_handler, store_handler):
        self.host = host
        self.port = port

        AccessHTTPServer.STATIC_PATH = static_path

        AccessHTTPServer.FETCH_HANDLER = fetch_handler
        fetch_processor = FetchCommunicationService.Processor(fetch_handler)
        fetch_pfactory = TJSONProtocol.TJSONProtocolFactory()
        AccessHTTPServer.FETCH_TSERVER = TServer.TServer(
            fetch_processor, None, None, None, fetch_pfactory, fetch_pfactory)

        AccessHTTPServer.STORE_HANDLER = store_handler
        store_processor = StoreCommunicationService.Processor(store_handler)
        store_pfactory = TJSONProtocol.TJSONProtocolFactory()
        AccessHTTPServer.STORE_TSERVER = TServer.TServer(
            store_processor, None, None, None, store_pfactory, store_pfactory)

    def serve(self):
        bottle.run(host=self.host, port=self.port)


@bottle.post('/fetch_http_endpoint/')
def fetch_http_endpoint():
    return thrift_endpoint(AccessHTTPServer.FETCH_TSERVER)


@bottle.post('/store_http_endpoint/')
def store_http_endpoint():
    return thrift_endpoint(AccessHTTPServer.STORE_TSERVER)


@bottle.route('/<filepath:path>')
def server_static(filepath):
    return bottle.static_file(filepath, root=AccessHTTPServer.STATIC_PATH)


def thrift_endpoint(tserver):
    """Thrift RPC endpoint
    """
    itrans = TTransport.TFileObjectTransport(bottle.request.body)
    itrans = TTransport.TBufferedTransport(
        itrans, int(bottle.request.headers['Content-Length']))
    otrans = TTransport.TMemoryBuffer()

    iprot = tserver.inputProtocolFactory.getProtocol(itrans)
    oprot = tserver.outputProtocolFactory.getProtocol(otrans)

    tserver.processor.process(iprot, oprot)
    bytestring = otrans.getvalue()

    headers = dict()
    headers['Content-Length'] = len(bytestring)
    headers['Content-Type'] = "application/x-thrift"
    return bottle.HTTPResponse(bytestring, **headers)


def main():
    set_stdout_encoding()

    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description=''
    )
    parser.add_argument('fetch_source')
    parser.add_argument('--host', default='localhost',
                        help='Host interface to listen on')
    parser.add_argument('-p', '--port', type=int, default=8080)
    parser.add_argument('-l', '--loglevel', '--log-level',
                        help='Logging verbosity level threshold (to stderr)',
                        default='info')
    parser.add_argument('--static-path', default='.',
                        help='Path where HTML files are stored')
    parser.add_argument('--store-path', default='.',
                        help='Path where Communications should be stored')
    parser.add_argument('--max-file-size', type=str, default='1GiB',
                        help="Maximum size of (non-ZIP) files that can be read into memory "
                        "(e.g. '2G', '300MB')")
    args = parser.parse_args()

    logging.basicConfig(format='%(asctime)-15s %(levelname)s: %(message)s',
                        level=args.loglevel.upper())

    comm_container = {}
    if os.path.isdir(args.fetch_source):
        comm_container = DirectoryBackedCommunicationContainer(args.fetch_source)
    elif zipfile.is_zipfile(args.fetch_source):
        comm_container = ZipFileBackedCommunicationContainer(args.fetch_source)
    else:
        max_file_size = humanfriendly.parse_size(args.max_file_size, binary=True)
        comm_container = MemoryBackedCommunicationContainer(args.fetch_source,
                                                            max_file_size=max_file_size)

    fetch_handler = CommunicationContainerFetchHandler(comm_container)
    store_handler = DirectoryBackedStoreHandler(args.store_path)

    logging.info('Fetch endpoint: http://{}:{}/fetch_http_endpoint/'.format(args.host, args.port))
    logging.info('Store endpoint: http://{}:{}/store_http_endpoint/'.format(args.host, args.port))

    ahs = AccessHTTPServer(args.host, args.port, args.static_path, fetch_handler, store_handler)
    ahs.serve()


if __name__ == '__main__':
    main()



from __future__ import unicode_literals
from datetime import datetime
import random
import string

from pytest import raises
from mock import Mock, sentinel, patch

from concrete.util import (
    get_index_of_tool, datetime_to_timestamp, now_timestamp,
    timestamp_to_datetime, get_annotation_field, filter_annotations,
    ZeroAnnotationsError, MultipleAnnotationsError, filter_annotations_json,
    tool_to_filter
)
from concrete import AnnotationMetadata


class HasMetadata(object):
    def __init__(self, tool=None):
        self.metadata = AnnotationMetadata(
            tool=HasMetadata.gen_tool(tool),
            timestamp=0,
        )

    @classmethod
    def gen_tool(cls, tool=None, size=6,
                 chars=string.ascii_uppercase + string.digits):
        return tool if (tool is not None) else \
            u''.join(random.choice(chars) for _ in range(size))


def test_get_index_of_tool_none_list():
    assert get_index_of_tool(None, None) == -1
    assert get_index_of_tool(None, "") == -1
    assert get_index_of_tool(None, "My awesome tool") == -1


def test_get_index_of_tool_empty_list():
    assert get_index_of_tool([], None) == -1
    assert get_index_of_tool([], "") == -1
    assert get_index_of_tool([], "My awesome tool") == -1


def test_get_index_of_tool_nonempty_list_contains():
    lst = [
        HasMetadata("My awesome tool"),
        HasMetadata("My awesome tool--new"),
        HasMetadata()
    ]
    assert get_index_of_tool(lst, None) == 0
    assert get_index_of_tool(lst, "") == -1
    assert get_index_of_tool(lst, "My awesome tool") == 0
    lst[2] = lst[0]
    lst[0] = HasMetadata()
    assert get_index_of_tool(lst, None) == 0
    assert get_index_of_tool(lst, "") == -1
    assert get_index_of_tool(lst, "My awesome tool") == 2


def test_get_index_of_tool_nonempty_list_no_contains():
    lst = [
        HasMetadata(),
        HasMetadata(),
        HasMetadata()
    ]
    assert get_index_of_tool(lst, None) == 0
    assert get_index_of_tool(lst, "") == -1
    assert get_index_of_tool(lst, "My awesome tool") == -1


def test_datetime_to_timestamp():
    assert datetime_to_timestamp(datetime(1970, 1, 1)) == 0
    assert datetime_to_timestamp(datetime(1970, 1, 1, 0, 0, 47)) == 47


def test_timestamp_to_datetime():
    assert timestamp_to_datetime(0) == datetime(1970, 1, 1)
    assert timestamp_to_datetime(47) == datetime(1970, 1, 1, 0, 0, 47)


def test_now_timestamp():
    assert now_timestamp() > datetime_to_timestamp(datetime(2000, 1, 1, 0, 0, 0))


def test_get_annotation_field_kBest():
    annotation = Mock(metadata=Mock(kBest=4))
    assert get_annotation_field(annotation, 'kBest') == 4


def test_get_annotation_field_tool():
    annotation = Mock(metadata=Mock(tool='goldenhorse'))
    assert get_annotation_field(annotation, 'tool') == 'goldenhorse'


def test_get_annotation_field_timestamp():
    annotation = Mock(metadata=Mock(timestamp=4))
    assert get_annotation_field(annotation, 'timestamp') == 4


def test_get_annotation_field_invalid():
    annotation = Mock(metadata=Mock())
    with raises(ValueError):
        get_annotation_field(annotation, 'foobar')


def test_filter_annotations_noop():
    assert filter_annotations([
        sentinel.annotation0,
        sentinel.annotation1,
        sentinel.annotation2,
    ]) == [
        sentinel.annotation0,
        sentinel.annotation1,
        sentinel.annotation2,
    ]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 4,
            (sentinel.annotation0, 'bar'): 3,
            (sentinel.annotation1, 'bar'): 4,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4}
    ) == [sentinel.annotation1]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_zero(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 4,
            (sentinel.annotation0, 'bar'): 3,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4}
    ) == []


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4}
    ) == [sentinel.annotation0, sentinel.annotation2]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_reverse(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_reverse=True
    ) == [sentinel.annotation3, sentinel.annotation2, sentinel.annotation0]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_sort(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
            (sentinel.annotation0, 'baz'): 2,
            (sentinel.annotation1, 'baz'): 0,
            (sentinel.annotation2, 'baz'): 1,
            (sentinel.annotation3, 'baz'): 3,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_field='baz'
    ) == [sentinel.annotation2, sentinel.annotation0, sentinel.annotation3]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_sort_reverse(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
            (sentinel.annotation0, 'baz'): 2,
            (sentinel.annotation1, 'baz'): 0,
            (sentinel.annotation2, 'baz'): 1,
            (sentinel.annotation3, 'baz'): 3,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_field='baz',
        sort_reverse=True
    ) == [sentinel.annotation3, sentinel.annotation0, sentinel.annotation2]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_raise(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    with raises(MultipleAnnotationsError):
        filter_annotations(
            [
                sentinel.annotation0,
                sentinel.annotation1,
                sentinel.annotation2,
            ],
            filter_fields={'foo': 3, 'bar': 4},
            action_if_multiple='raise'
        )


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_zero_raise(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 4,
            (sentinel.annotation0, 'bar'): 3,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    with raises(ZeroAnnotationsError):
        filter_annotations(
            [
                sentinel.annotation0,
                sentinel.annotation1,
                sentinel.annotation2,
            ],
            filter_fields={'foo': 3, 'bar': 4},
            action_if_zero='raise'
        )


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_zero_raise_with_multiple(
        mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        action_if_zero='raise'
    ) == [sentinel.annotation0, sentinel.annotation2]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_raise_with_zero(
        mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 4,
            (sentinel.annotation0, 'bar'): 3,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        action_if_multiple='raise'
    ) == []


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_zero_raise_with_one(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 4,
            (sentinel.annotation0, 'bar'): 3,
            (sentinel.annotation1, 'bar'): 4,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4}
    ) == [sentinel.annotation1]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_raise_with_one(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 4,
            (sentinel.annotation0, 'bar'): 3,
            (sentinel.annotation1, 'bar'): 4,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4}
    ) == [sentinel.annotation1]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_first(
        mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        action_if_multiple='first'
    ) == [sentinel.annotation0]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_last(
        mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        action_if_multiple='last'
    ) == [sentinel.annotation2]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_sort_first(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
            (sentinel.annotation0, 'baz'): 2,
            (sentinel.annotation1, 'baz'): 0,
            (sentinel.annotation2, 'baz'): 1,
            (sentinel.annotation3, 'baz'): 3,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_field='baz',
        action_if_multiple='first'
    ) == [sentinel.annotation2]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_sort_last(mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
            (sentinel.annotation0, 'baz'): 2,
            (sentinel.annotation1, 'baz'): 0,
            (sentinel.annotation2, 'baz'): 1,
            (sentinel.annotation3, 'baz'): 3,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_field='baz',
        action_if_multiple='last'
    ) == [sentinel.annotation3]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_sort_reverse_first(
        mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
            (sentinel.annotation0, 'baz'): 2,
            (sentinel.annotation1, 'baz'): 0,
            (sentinel.annotation2, 'baz'): 1,
            (sentinel.annotation3, 'baz'): 3,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_field='baz',
        sort_reverse=True,
        action_if_multiple='first'
    ) == [sentinel.annotation3]


@patch('concrete.util.metadata.get_annotation_field')
def test_filter_annotations_multiple_sort_reverse_last(
        mock_get_annotation_field):
    def _mock_get_annotation_field(annotation, field):
        return {
            (sentinel.annotation0, 'foo'): 3,
            (sentinel.annotation1, 'foo'): 3,
            (sentinel.annotation2, 'foo'): 3,
            (sentinel.annotation3, 'foo'): 3,
            (sentinel.annotation0, 'bar'): 4,
            (sentinel.annotation1, 'bar'): 3,
            (sentinel.annotation2, 'bar'): 4,
            (sentinel.annotation3, 'bar'): 4,
            (sentinel.annotation0, 'baz'): 2,
            (sentinel.annotation1, 'baz'): 0,
            (sentinel.annotation2, 'baz'): 1,
            (sentinel.annotation3, 'baz'): 3,
        }[(annotation, field)]
    mock_get_annotation_field.side_effect = _mock_get_annotation_field

    assert filter_annotations(
        [
            sentinel.annotation0,
            sentinel.annotation1,
            sentinel.annotation2,
            sentinel.annotation3,
        ],
        filter_fields={'foo': 3, 'bar': 4},
        sort_field='baz',
        sort_reverse=True,
        action_if_multiple='last'
    ) == [sentinel.annotation2]


@patch('concrete.util.metadata.filter_annotations')
def test_filter_annotations_json(mock_filter_annotations):
    mock_filter_annotations.side_effect = [sentinel.return_value]
    assert filter_annotations_json(
        sentinel.annotations,
        '{"foo": 47, "baz": ["hello", "world"]}'
    ) == sentinel.return_value
    mock_filter_annotations.assert_called_once_with(
        sentinel.annotations,
        foo=47,
        baz=['hello', 'world'],
    )


def test_tool_to_filter():
    assert tool_to_filter(None, None) is None
    assert tool_to_filter(None, sentinel.explicit_filter) == sentinel.explicit_filter

    annotation_filter = tool_to_filter(sentinel.tool, None)
    annotations = [
        Mock(metadata=Mock(tool=sentinel.tool)),
        Mock(metadata=Mock(tool=sentinel.other_tool)),
        Mock(metadata=Mock(tool=sentinel.tool)),
    ]
    assert annotation_filter(annotations) == [annotations[0], annotations[2]]

    with raises(ValueError):
        tool_to_filter(sentinel.tool, sentinel.explicit_filter)



"""Communication Containers - mapping Communication IDs to Communications

Classes that behave like a read-only dictionary (implementing Python's
`collections.Mapping` interface) and map Communication ID strings to
Communications.

The classes abstract away the storage backend.  If you need to
optimize for performance, you may not want to use a dictionary
abstraction that retrieves one Communication at a time.
"""
from __future__ import unicode_literals

import collections
import gzip
import logging
import os
import zipfile

import humanfriendly

from ..access.ttypes import FetchRequest
from .access import (
    prefix_s3_key, unprefix_s3_key, DEFAULT_S3_KEY_PREFIX_LEN,
)
from .access_wrapper import FetchCommunicationClientWrapper
from .file_io import (
    CommunicationReader,
    read_communication_from_file)
from .mem_io import read_communication_from_buffer


class DirectoryBackedCommunicationContainer(collections.Mapping):
    """Maps Comm IDs to Comms, retrieving Comms from the filesystem

    `DirectoryBackedCommunicationContainer` instances behave as
    dict-like data structures that map Communication IDs to
    Communications.  Communications are lazily retrieved from the
    filesystem.

    Upon initialization, a `DirectoryBackedCommunicationContainer`
    instance will (recursively) search `directory_path` for any files
    that end with the specified `comm_extensions`.  Files with
    matching extensions are assumed to be Communication files whose
    filename (sans extension) is the file's Communication ID.  So, for
    example, a file named 'XIN_ENG_20101212.0120.concrete' is assumed
    to be a Communication file with a Communication ID of
    'XIN_ENG_20101212.0120'.

    Files with the extension `.gz` will be decompressed using gzip.

    A `DirectoryBackedCommunicationsContainer` will not be able to find
    any files that are added to `directory_path` after the container
    was initialized.
    """

    def __init__(self, directory_path,
                 comm_extensions=['.comm', '.concrete', '.gz']):
        """
        Args:
             directory_path (str): Path to directory containing Communications files
             comm_extensions (str[]): List of strings specifying filename extensions
                                         to be associated with Communications
        """
        self.comm_id_to_comm_path = {}

        logging.info("Caching names of files with extensions [%s] in '%s'" %
                     (', '.join(comm_extensions), directory_path))

        # Recursively traverse subdirectories
        for root, dirs, files in os.walk(directory_path):
            for basename in files:
                (comm_id, extension) = os.path.splitext(basename)
                if extension in comm_extensions:
                    comm_filename = os.path.join(root, basename)
                    self.comm_id_to_comm_path[comm_id] = comm_filename

        logging.info("Finished caching %d Communication filenames in '%s'" %
                     (len(self.comm_id_to_comm_path), directory_path))

    def __getitem__(self, communication_id):
        if communication_id in self.comm_id_to_comm_path:
            comm_path = self.comm_id_to_comm_path[communication_id]
            logging.debug("Reading Communication with ID '%s'" % comm_path)
            if not os.path.exists(comm_path):
                logging.error("Unable to find file with path '%s'" % comm_path)
                raise KeyError
            if os.path.splitext(comm_path)[1] == '.gz':
                with gzip.open(comm_path) as gzip_file:
                    buf = gzip_file.read()
                    comm = read_communication_from_buffer(buf)
            else:
                comm = read_communication_from_file(comm_path)
            return comm
        else:
            logging.debug('No Communication with ID: %s' % communication_id)
            raise KeyError

    def __iter__(self):
        return self.comm_id_to_comm_path.__iter__()

    def __len__(self):
        return len(self.comm_id_to_comm_path)


class FetchBackedCommunicationContainer(collections.Mapping):
    """Maps Comm IDs to Comms, retrieving Comms from a
    :mod:`.FetchCommunicationService` server

    `FetchBackedCommunicationContainer` instances behave as dict-like data
    structures that map Communication IDs to Communications.  Communications
    are lazily retrieved from a :mod:`.FetchCommunicationService`.

    If you need to retrieve large amounts of data from a
    :mod:`.FetchCommunicationService`, then you SHOULD NOT USE THIS CLASS.
    This class retrieves one Communication at a time using
    :mod:`.FetchCommunicationService`.
    """

    def __init__(self, host, port):
        """
        Args:
            host (str): Hostname of :mod:`.FetchCommunicationService` server
            port (int): Port # of :mod:`.FetchCommunicationService` server
        """
        self.host = host
        self.port = port

    def __getitem__(self, communication_id):
        with FetchCommunicationClientWrapper(self.host, self.port) as fc:
            fetch_result = fc.fetch(FetchRequest(communicationIds=[communication_id]))
            total_results = len(fetch_result.communications)
            if total_results == 0:
                raise KeyError
            elif total_results == 1:
                return fetch_result.communications[0]
            else:
                raise Exception("FetchBackedCommunicationContainer.__get_item__() "
                                "expected to receive 1 Communication, but instead "
                                "received %d Communications" % total_results)

    def __iter__(self):
        with FetchCommunicationClientWrapper(self.host, self.port) as fc:
            n = fc.getCommunicationCount()
            return fc.getCommunicationIDs(0, n).__iter__()

    def __len__(self):
        with FetchCommunicationClientWrapper(self.host, self.port) as fc:
            return fc.getCommunicationCount()


class MemoryBackedCommunicationContainer(collections.Mapping):
    """Maps Comm IDs to Comms by loading all Comms in file into memory

    `FetchBackedCommunicationContainer` instances behave as dict-like
    data structures that map Communication IDs to Communications.  All
    Communications in `communications_file` will be read into memory
    using a :class:`.CommunicationReader` instance.
    """

    def __init__(self, communications_file, max_file_size=1073741824):
        """
        Args:
            communications_file (str): String specifying name of Communications file
            max_file_size (int): Maximum file size, in bytes
        """
        self.comm_id_to_comm = {}

        comm_file_size = os.path.getsize(communications_file)
        if comm_file_size > max_file_size:
            raise Exception(
                ("MemoryBackedComunicationContainer will not open the file "
                 "'%s' because the file's size (%s) is larger than the "
                 "maximum specified file size of %s.  If you would like to "
                 "read a file this large into memory, you will need to "
                 "specify a larger value for the 'max_file_size'parameter.") %
                (communications_file,
                 humanfriendly.format_size(comm_file_size, binary=True),
                 humanfriendly.format_size(max_file_size, binary=True)))

        logging.info("Reading in Communications from file '%s'" %
                     communications_file)
        logging.debug("Communication IDs:")
        for (comm, _) in CommunicationReader(communications_file):
            self.comm_id_to_comm[comm.id] = comm
            logging.debug("  %s" % comm.id)
        logging.info("Finished reading communications.\n")

    def __getitem__(self, communication_id):
        return self.comm_id_to_comm[communication_id]

    def __iter__(self):
        return self.comm_id_to_comm.__iter__()

    def __len__(self):
        return len(self.comm_id_to_comm)


class ZipFileBackedCommunicationContainer(collections.Mapping):
    """Maps Comm IDs to Comms, retrieving Comms from a Zip file

    `ZipFileBackedCommunicationContainer` instances behave as dict-like
    data structures that map Communication IDs to Communications.
    Communications are lazily retrieved from a Zip file.
    """

    def __init__(self, zipfile_path, comm_extensions=['.comm', '.concrete']):
        """
        Args:
            zipfile_path (str): Path to Zip file containing Communications
            comm_extensions (str[]): List of strings specifying filename extensions
                                 associated with Communications
        """
        self.comm_id_to_filename = {}
        self.zipfile = zipfile.ZipFile(zipfile_path, 'r')
        for filename in self.zipfile.namelist():
            (comm_id, extension) = os.path.splitext(os.path.basename(filename))
            if extension in comm_extensions:
                self.comm_id_to_filename[comm_id] = filename

    def __exit__(self):
        self.zipfile.close()

    def __getitem__(self, communication_id):
        filename = self.comm_id_to_filename[communication_id]
        buf = self.zipfile.read(filename)
        comm = read_communication_from_buffer(buf)
        return comm

    def __iter__(self):
        return self.comm_id_to_filename.__iter__()

    def __len__(self):
        return len(self.comm_id_to_filename)


class RedisHashBackedCommunicationContainer(collections.Mapping):
    """
    Provides access to Communications stored in a Redis hash,
    assuming the key of each communication is its Communication id.

    `RedisHashBackedCommunicationContainer` instances behave as dict-like
    data structures that map Communication IDs to Communications.
    Communications are lazily retrieved from a Redis hash.
    """

    def __init__(self, redis_db, key):
        """
        Args:
            redis_db (redis.Redis): Redis database connection object
            key (str): Key in redis database where hash is located
        """
        self.redis_db = redis_db
        self.key = key

    def __getitem__(self, communication_id):
        buf = self.redis_db.hget(self.key, communication_id)
        if buf is None:
            raise KeyError
        comm = read_communication_from_buffer(buf)
        return comm

    def __contains__(self, communication_id):
        return self.redis_db.hexists(self.key, communication_id)

    def __iter__(self):
        return iter(self.redis_db.hkeys(self.key))

    def __len__(self):
        return self.redis_db.hlen(self.key)


class S3BackedCommunicationContainer(collections.Mapping):
    """
    Provides access to Communications stored in an AWS S3 bucket,
    assuming the key of each communication is its Communication id
    (optionally prefixed with a fixed-length, random-looking but
    deterministic hash to improve performance).

    `S3HashBackedCommunicationContainer` instances behave as dict-like
    data structures that map Communication IDs (with or without
    prefixes) to Communications.  Communications are lazily retrieved
    from an S3 bucket.

    References:
        http://docs.aws.amazon.com/AmazonS3/latest/dev/request-rate-perf-considerations.html
    """

    def __init__(self, bucket, prefix_len=DEFAULT_S3_KEY_PREFIX_LEN):
        """
        Args:
            bucket (boto.s3.bucket.Bucket): S3 bucket object
            prefix_len (int): length of prefix in each
                Communication's key in the bucket.  This number of
                characters will be removed from the beginning of the
                key to determine the Communication id (without incurring
                the cost of fetching and deserializing the
                Communication).  A prefix
                enables S3 to better partition the bucket contents,
                yielding higher performance and a lower chance of
                getting rate-limited by AWS.
        """
        self.bucket = bucket
        self.prefix_len = prefix_len

    def __getitem__(self, communication_id):
        prefixed_key_str = prefix_s3_key(communication_id, self.prefix_len)
        key = self.bucket.get_key(prefixed_key_str)
        if key is None:
            raise KeyError
        buf = key.get_contents_as_string()
        comm = read_communication_from_buffer(buf)
        return comm

    def __contains__(self, communication_id):
        prefixed_key_str = prefix_s3_key(communication_id, self.prefix_len)
        return self.bucket.get_key(prefixed_key_str) is not None

    def __iter__(self):
        return iter(
            unprefix_s3_key(key.name, self.prefix_len)
            for key in self.bucket.list())

    def __len__(self):
        n = 0
        for comm_id in self:
            n += 1
        return n



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.services.Service
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(concrete.services.Service.Iface):
    def search(self, query):
        """
        Perform a search specified by the query

        Parameters:
         - query
        """
        pass

    def getCapabilities(self):
        """
        Get a list of search type-language pairs
        """
        pass

    def getCorpora(self):
        """
        Get a corpus list from the search provider
        """
        pass


class Client(concrete.services.Service.Client, Iface):
    def __init__(self, iprot, oprot=None):
        concrete.services.Service.Client.__init__(self, iprot, oprot)

    def search(self, query):
        """
        Perform a search specified by the query

        Parameters:
         - query
        """
        self.send_search(query)
        return self.recv_search()

    def send_search(self, query):
        self._oprot.writeMessageBegin('search', TMessageType.CALL, self._seqid)
        args = search_args()
        args.query = query
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_search(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = search_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "search failed: unknown result")

    def getCapabilities(self):
        """
        Get a list of search type-language pairs
        """
        self.send_getCapabilities()
        return self.recv_getCapabilities()

    def send_getCapabilities(self):
        self._oprot.writeMessageBegin('getCapabilities', TMessageType.CALL, self._seqid)
        args = getCapabilities_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getCapabilities(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getCapabilities_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getCapabilities failed: unknown result")

    def getCorpora(self):
        """
        Get a corpus list from the search provider
        """
        self.send_getCorpora()
        return self.recv_getCorpora()

    def send_getCorpora(self):
        self._oprot.writeMessageBegin('getCorpora', TMessageType.CALL, self._seqid)
        args = getCorpora_args()
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_getCorpora(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = getCorpora_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.ex is not None:
            raise result.ex
        raise TApplicationException(TApplicationException.MISSING_RESULT, "getCorpora failed: unknown result")


class Processor(concrete.services.Service.Processor, Iface, TProcessor):
    def __init__(self, handler):
        concrete.services.Service.Processor.__init__(self, handler)
        self._processMap["search"] = Processor.process_search
        self._processMap["getCapabilities"] = Processor.process_getCapabilities
        self._processMap["getCorpora"] = Processor.process_getCorpora

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_search(self, seqid, iprot, oprot):
        args = search_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = search_result()
        try:
            result.success = self._handler.search(args.query)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("search", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getCapabilities(self, seqid, iprot, oprot):
        args = getCapabilities_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getCapabilities_result()
        try:
            result.success = self._handler.getCapabilities()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getCapabilities", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_getCorpora(self, seqid, iprot, oprot):
        args = getCorpora_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = getCorpora_result()
        try:
            result.success = self._handler.getCorpora()
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("getCorpora", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class search_args(object):
    """
    Attributes:
     - query
    """


    def __init__(self, query=None,):
        self.query = query

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.query = SearchQuery()
                    self.query.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('search_args')
        if self.query is not None:
            oprot.writeFieldBegin('query', TType.STRUCT, 1)
            self.query.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(search_args)
search_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'query', [SearchQuery, None], None, ),  # 1
)


class search_result(object):
    """
    Attributes:
     - success
     - ex
    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = SearchResult()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException()
                    self.ex.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('search_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(search_result)
search_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [SearchResult, None], None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getCapabilities_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getCapabilities_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getCapabilities_args)
getCapabilities_args.thrift_spec = (
)


class getCapabilities_result(object):
    """
    Attributes:
     - success
     - ex
    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.LIST:
                    self.success = []
                    (_etype31, _size28) = iprot.readListBegin()
                    for _i32 in range(_size28):
                        _elem33 = SearchCapability()
                        _elem33.read(iprot)
                        self.success.append(_elem33)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException()
                    self.ex.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getCapabilities_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.LIST, 0)
            oprot.writeListBegin(TType.STRUCT, len(self.success))
            for iter34 in self.success:
                iter34.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getCapabilities_result)
getCapabilities_result.thrift_spec = (
    (0, TType.LIST, 'success', (TType.STRUCT, [SearchCapability, None], False), None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class getCorpora_args(object):


    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getCorpora_args')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getCorpora_args)
getCorpora_args.thrift_spec = (
)


class getCorpora_result(object):
    """
    Attributes:
     - success
     - ex
    """


    def __init__(self, success=None, ex=None,):
        self.success = success
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.LIST:
                    self.success = []
                    (_etype38, _size35) = iprot.readListBegin()
                    for _i39 in range(_size35):
                        _elem40 = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                        self.success.append(_elem40)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException()
                    self.ex.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('getCorpora_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.LIST, 0)
            oprot.writeListBegin(TType.STRING, len(self.success))
            for iter41 in self.success:
                oprot.writeString(iter41.encode('utf-8') if sys.version_info[0] == 2 else iter41)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(getCorpora_result)
getCorpora_result.thrift_spec = (
    (0, TType.LIST, 'success', (TType.STRING, 'UTF8', False), None, ),  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)
fix_spec(all_structs)
del all_structs




# -*- coding: utf-8 -*-


from __future__ import unicode_literals
from pytest import fixture, mark
import io
import os
import sys
import json
from subprocess import Popen, PIPE
from tempfile import mkstemp


SERIF_TEXT = '''\
<DOC id="dog-bites-man" type="other">
<HEADLINE>
Dog Bites Man
</HEADLINE>
<TEXT>
<P>
John Smith, manager of ACMÉ INC, was bit by a dog on March 10th, 2013.
</P>
<P>
He died!
</P>
<P>
John's daughter Mary expressed sorrow.
</P>
</TEXT>
</DOC>
'''

LES_DEUX_TEXT = '''\
Madame Magloire comprit, et elle alla chercher sur la cheminée de la \
chambre à coucher de monseigneur les deux chandeliers d'argent \
qu'elle posa sur la table tout allumés.

—Monsieur le curé, dit l'homme, vous êtes bon. Vous ne me méprisez \
pas. Vous me recevez chez vous. Vous allumez vos cierges pour moi. \
Je ne vous ai pourtant pas caché d'où je viens et que je suis un homme \
malheureux.
'''


def assert_json_protocol_ok(json_path, expected_text):
    with io.open(json_path, encoding='utf-8') as f:
        json_obj = json.load(f)
        assert json_obj['4']['str'] == expected_text


def assert_simple_json_protocol_ok(json_path, expected_text):
    with io.open(json_path, encoding='utf-8') as f:
        json_obj = json.load(f)
        assert json_obj['text'] == expected_text


@fixture
def output_file(request):
    (fd, path) = mkstemp()
    os.close(fd)

    def _remove():
        if os.path.exists(path):
            os.remove(path)

    request.addfinalizer(_remove)
    return path


@mark.parametrize('args,text,assertion', [
    (['tests/testdata/les-deux-chandeliers.concrete'],
     LES_DEUX_TEXT, assert_simple_json_protocol_ok),
    (['--protocol', 'simple', 'tests/testdata/les-deux-chandeliers.concrete'],
     LES_DEUX_TEXT, assert_simple_json_protocol_ok),
    (['--protocol', 'TJSONProtocol', 'tests/testdata/les-deux-chandeliers.concrete'],
     LES_DEUX_TEXT, assert_json_protocol_ok),
    (['tests/testdata/serif_dog-bites-man.concrete'],
     SERIF_TEXT, assert_simple_json_protocol_ok),
    (['--protocol', 'simple', 'tests/testdata/serif_dog-bites-man.concrete'],
     SERIF_TEXT, assert_simple_json_protocol_ok),
    (['--protocol', 'TJSONProtocol', 'tests/testdata/serif_dog-bites-man.concrete'],
     SERIF_TEXT, assert_json_protocol_ok),
])
def test_concrete2json(output_file, args, text, assertion):
    p = Popen([
        sys.executable,
        'scripts/concrete2json.py',
    ] + args + [
        output_file
    ], stdout=PIPE, stderr=PIPE)
    (stdout, stderr) = p.communicate()
    assert stdout == b''
    assert stderr == b''
    assert p.returncode == 0

    assertion(output_file, text)



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.services.Service
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(concrete.services.Service.Iface):
    def startFeedback(self, results):
        """
        Start providing feedback for the specified SearchResults.
        This causes the search and its results to be persisted.

        Parameters:
         - results
        """
        pass

    def addCommunicationFeedback(self, searchResultsId, communicationId, feedback):
        """
        Provide feedback on the relevance of a particular communication to a search

        Parameters:
         - searchResultsId
         - communicationId
         - feedback
        """
        pass

    def addSentenceFeedback(self, searchResultsId, communicationId, sentenceId, feedback):
        """
        Provide feedback on the relevance of a particular sentence to a search

        Parameters:
         - searchResultsId
         - communicationId
         - sentenceId
         - feedback
        """
        pass


class Client(concrete.services.Service.Client, Iface):
    def __init__(self, iprot, oprot=None):
        concrete.services.Service.Client.__init__(self, iprot, oprot)

    def startFeedback(self, results):
        """
        Start providing feedback for the specified SearchResults.
        This causes the search and its results to be persisted.

        Parameters:
         - results
        """
        self.send_startFeedback(results)
        self.recv_startFeedback()

    def send_startFeedback(self, results):
        self._oprot.writeMessageBegin('startFeedback', TMessageType.CALL, self._seqid)
        args = startFeedback_args()
        args.results = results
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_startFeedback(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = startFeedback_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.ex is not None:
            raise result.ex
        return

    def addCommunicationFeedback(self, searchResultsId, communicationId, feedback):
        """
        Provide feedback on the relevance of a particular communication to a search

        Parameters:
         - searchResultsId
         - communicationId
         - feedback
        """
        self.send_addCommunicationFeedback(searchResultsId, communicationId, feedback)
        self.recv_addCommunicationFeedback()

    def send_addCommunicationFeedback(self, searchResultsId, communicationId, feedback):
        self._oprot.writeMessageBegin('addCommunicationFeedback', TMessageType.CALL, self._seqid)
        args = addCommunicationFeedback_args()
        args.searchResultsId = searchResultsId
        args.communicationId = communicationId
        args.feedback = feedback
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_addCommunicationFeedback(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = addCommunicationFeedback_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.ex is not None:
            raise result.ex
        return

    def addSentenceFeedback(self, searchResultsId, communicationId, sentenceId, feedback):
        """
        Provide feedback on the relevance of a particular sentence to a search

        Parameters:
         - searchResultsId
         - communicationId
         - sentenceId
         - feedback
        """
        self.send_addSentenceFeedback(searchResultsId, communicationId, sentenceId, feedback)
        self.recv_addSentenceFeedback()

    def send_addSentenceFeedback(self, searchResultsId, communicationId, sentenceId, feedback):
        self._oprot.writeMessageBegin('addSentenceFeedback', TMessageType.CALL, self._seqid)
        args = addSentenceFeedback_args()
        args.searchResultsId = searchResultsId
        args.communicationId = communicationId
        args.sentenceId = sentenceId
        args.feedback = feedback
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_addSentenceFeedback(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = addSentenceFeedback_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.ex is not None:
            raise result.ex
        return


class Processor(concrete.services.Service.Processor, Iface, TProcessor):
    def __init__(self, handler):
        concrete.services.Service.Processor.__init__(self, handler)
        self._processMap["startFeedback"] = Processor.process_startFeedback
        self._processMap["addCommunicationFeedback"] = Processor.process_addCommunicationFeedback
        self._processMap["addSentenceFeedback"] = Processor.process_addSentenceFeedback

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_startFeedback(self, seqid, iprot, oprot):
        args = startFeedback_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = startFeedback_result()
        try:
            self._handler.startFeedback(args.results)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("startFeedback", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_addCommunicationFeedback(self, seqid, iprot, oprot):
        args = addCommunicationFeedback_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = addCommunicationFeedback_result()
        try:
            self._handler.addCommunicationFeedback(args.searchResultsId, args.communicationId, args.feedback)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("addCommunicationFeedback", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_addSentenceFeedback(self, seqid, iprot, oprot):
        args = addSentenceFeedback_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = addSentenceFeedback_result()
        try:
            self._handler.addSentenceFeedback(args.searchResultsId, args.communicationId, args.sentenceId, args.feedback)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except concrete.services.ttypes.ServicesException as ex:
            msg_type = TMessageType.REPLY
            result.ex = ex
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("addSentenceFeedback", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class startFeedback_args(object):
    """
    Attributes:
     - results
    """


    def __init__(self, results=None,):
        self.results = results

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.results = SearchResult()
                    self.results.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('startFeedback_args')
        if self.results is not None:
            oprot.writeFieldBegin('results', TType.STRUCT, 1)
            self.results.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(startFeedback_args)
startFeedback_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'results', [SearchResult, None], None, ),  # 1
)


class startFeedback_result(object):
    """
    Attributes:
     - ex
    """


    def __init__(self, ex=None,):
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException()
                    self.ex.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('startFeedback_result')
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(startFeedback_result)
startFeedback_result.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class addCommunicationFeedback_args(object):
    """
    Attributes:
     - searchResultsId
     - communicationId
     - feedback
    """


    def __init__(self, searchResultsId=None, communicationId=None, feedback=None,):
        self.searchResultsId = searchResultsId
        self.communicationId = communicationId
        self.feedback = feedback

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.searchResultsId = concrete.uuid.ttypes.UUID()
                    self.searchResultsId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.communicationId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I32:
                    self.feedback = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('addCommunicationFeedback_args')
        if self.searchResultsId is not None:
            oprot.writeFieldBegin('searchResultsId', TType.STRUCT, 1)
            self.searchResultsId.write(oprot)
            oprot.writeFieldEnd()
        if self.communicationId is not None:
            oprot.writeFieldBegin('communicationId', TType.STRING, 2)
            oprot.writeString(self.communicationId.encode('utf-8') if sys.version_info[0] == 2 else self.communicationId)
            oprot.writeFieldEnd()
        if self.feedback is not None:
            oprot.writeFieldBegin('feedback', TType.I32, 3)
            oprot.writeI32(self.feedback)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(addCommunicationFeedback_args)
addCommunicationFeedback_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'searchResultsId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRING, 'communicationId', 'UTF8', None, ),  # 2
    (3, TType.I32, 'feedback', None, None, ),  # 3
)


class addCommunicationFeedback_result(object):
    """
    Attributes:
     - ex
    """


    def __init__(self, ex=None,):
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException()
                    self.ex.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('addCommunicationFeedback_result')
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(addCommunicationFeedback_result)
addCommunicationFeedback_result.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)


class addSentenceFeedback_args(object):
    """
    Attributes:
     - searchResultsId
     - communicationId
     - sentenceId
     - feedback
    """


    def __init__(self, searchResultsId=None, communicationId=None, sentenceId=None, feedback=None,):
        self.searchResultsId = searchResultsId
        self.communicationId = communicationId
        self.sentenceId = sentenceId
        self.feedback = feedback

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.searchResultsId = concrete.uuid.ttypes.UUID()
                    self.searchResultsId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.communicationId = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    self.sentenceId = concrete.uuid.ttypes.UUID()
                    self.sentenceId.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    self.feedback = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('addSentenceFeedback_args')
        if self.searchResultsId is not None:
            oprot.writeFieldBegin('searchResultsId', TType.STRUCT, 1)
            self.searchResultsId.write(oprot)
            oprot.writeFieldEnd()
        if self.communicationId is not None:
            oprot.writeFieldBegin('communicationId', TType.STRING, 2)
            oprot.writeString(self.communicationId.encode('utf-8') if sys.version_info[0] == 2 else self.communicationId)
            oprot.writeFieldEnd()
        if self.sentenceId is not None:
            oprot.writeFieldBegin('sentenceId', TType.STRUCT, 3)
            self.sentenceId.write(oprot)
            oprot.writeFieldEnd()
        if self.feedback is not None:
            oprot.writeFieldBegin('feedback', TType.I32, 4)
            oprot.writeI32(self.feedback)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(addSentenceFeedback_args)
addSentenceFeedback_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'searchResultsId', [concrete.uuid.ttypes.UUID, None], None, ),  # 1
    (2, TType.STRING, 'communicationId', 'UTF8', None, ),  # 2
    (3, TType.STRUCT, 'sentenceId', [concrete.uuid.ttypes.UUID, None], None, ),  # 3
    (4, TType.I32, 'feedback', None, None, ),  # 4
)


class addSentenceFeedback_result(object):
    """
    Attributes:
     - ex
    """


    def __init__(self, ex=None,):
        self.ex = ex

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.ex = concrete.services.ttypes.ServicesException()
                    self.ex.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('addSentenceFeedback_result')
        if self.ex is not None:
            oprot.writeFieldBegin('ex', TType.STRUCT, 1)
            self.ex.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(addSentenceFeedback_result)
addSentenceFeedback_result.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'ex', [concrete.services.ttypes.ServicesException, None], None, ),  # 1
)
fix_spec(all_structs)
del all_structs




# -*- coding: utf-8 -*-


from __future__ import unicode_literals
from pytest import fixture, mark
from concrete.util import CommunicationReader
from concrete.validate import validate_communication
import os
import sys
from subprocess import Popen, PIPE
from tempfile import mkstemp


@fixture
def text():
    return u'''\
Madame Magloire comprit, et elle alla chercher sur la cheminée de la \
chambre à coucher de monseigneur les deux chandeliers d'argent \
qu'elle posa sur la table tout allumés.

—Monsieur le curé, dit l'homme, vous êtes bon. Vous ne me méprisez \
pas. Vous me recevez chez vous. Vous allumez vos cierges pour moi. \
Je ne vous ai pourtant pas caché d'où je viens et que je suis un homme \
malheureux.
'''


@fixture
def output_file(request):
    (fd, path) = mkstemp()
    os.close(fd)

    def _remove():
        if os.path.exists(path):
            os.remove(path)

    request.addfinalizer(_remove)
    return path


def test_create_comm(output_file, text):
    p = Popen([
        sys.executable,
        'scripts/create-comm.py',
        'tests/testdata/les-deux-chandeliers.txt',
        output_file
    ], stdout=PIPE, stderr=PIPE)
    (stdout, stderr) = p.communicate()
    assert p.returncode == 0

    reader = CommunicationReader(output_file)
    it = iter(reader)

    (comm, _) = next(it)
    assert comm.id == 'tests/testdata/les-deux-chandeliers.txt'
    assert validate_communication(comm)
    assert comm.text == text
    assert comm.sectionList is None

    try:
        next(it)
    except StopIteration:
        pass
    else:
        assert False


@mark.posix
def test_create_comm_stdin(output_file, text):
    p = Popen([
        sys.executable,
        'scripts/create-comm.py',
        '-',
        output_file
    ], stdin=PIPE, stdout=PIPE, stderr=PIPE)
    with open('tests/testdata/les-deux-chandeliers.txt', 'rb') as f:
        (stdout, stderr) = p.communicate(f.read())
    assert p.returncode == 0

    reader = CommunicationReader(output_file)
    it = iter(reader)

    (comm, _) = next(it)
    assert comm.id == '/dev/fd/0'
    assert validate_communication(comm)
    assert comm.text == text
    assert comm.sectionList is None

    try:
        next(it)
    except StopIteration:
        pass
    else:
        assert False


@mark.posix
def test_create_comm_stdout(output_file, text):
    p = Popen([
        sys.executable,
        'scripts/create-comm.py',
        'tests/testdata/les-deux-chandeliers.txt',
        '-'
    ], stdout=PIPE, stderr=PIPE)
    (stdout, stderr) = p.communicate()
    assert p.returncode == 0

    with open(output_file, 'wb') as f:
        f.write(stdout)

    reader = CommunicationReader(output_file)
    it = iter(reader)

    (comm, _) = next(it)
    assert comm.id == 'tests/testdata/les-deux-chandeliers.txt'
    assert validate_communication(comm)
    assert comm.text == text
    assert comm.sectionList is None

    try:
        next(it)
    except StopIteration:
        pass
    else:
        assert False


def test_create_comm_annotated(output_file, text):
    p = Popen([
        sys.executable,
        'scripts/create-comm.py',
        '--annotation-level', 'section',
        'tests/testdata/les-deux-chandeliers.txt',
        output_file
    ], stdout=PIPE, stderr=PIPE)
    (stdout, stderr) = p.communicate()
    assert p.returncode == 0

    reader = CommunicationReader(output_file)
    it = iter(reader)

    (comm, _) = next(it)
    assert comm.id == 'tests/testdata/les-deux-chandeliers.txt'
    assert validate_communication(comm)
    assert comm.text == text
    assert len(comm.sectionList) == 2

    try:
        next(it)
    except StopIteration:
        pass
    else:
        assert False
